#!/usr/bin/env python3

# Copyright (c) 2020 Taler Systems S.A.
# 
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted.
# 
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
# INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
# LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
# OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE.

import click
import random

def complete_iban(s):
    n = int("".join([str(int(x, 26)) for x in (s[4:] + s[0:2] + "00")]))
    c = 98 - (n % 97)
    return (s[:2] + str(c).ljust(2, "0") + s[4:]).upper()


@click.command()
def geniban():
    bban = "12345678"
    accno = "".join((str(random.randint(0, 9)) for _ in range(10)))
    iban = complete_iban("DE00" + bban + accno)
    print(iban)

if __name__ == '__main__':
    geniban()



#!/usr/bin/python3
# Clean testbench logs directory to only keep unique and useful files

import hashlib
from pathlib import Path
from string import whitespace

DIR = Path("test")


def rmtree(p: Path):
    """Recursively delete file or directory"""
    if p.exists():
        if p.is_dir():
            for child in p.iterdir():
                rmtree(child)
            p.rmdir()
        else:
            p.unlink()


def remove(p: Path, reason: str):
    """Announce and recursively remove file or directory"""
    print(f"rm {reason} {p}")
    rmtree(p)


content_hashes = set()


def rm_if_similar(p: Path, content: str):
    """Delete file if another file has the same content"""
    # Remove whitespace from file
    normalized = content.translate(str.maketrans("", "", whitespace))
    # Hash their content
    hash = hashlib.blake2b(normalized.encode(), usedforsecurity=False).hexdigest()
    if hash in content_hashes:
        remove(p, "similar")
    else:
        content_hashes.add(hash)


for platform in DIR.iterdir():
    if not platform.is_dir():
        continue
    for date in platform.iterdir():
        if not date.is_dir():
            continue
        for request in date.iterdir():
            payload_file_path = request.joinpath("payload.xml")
            payload_dir_path = request.joinpath("payload")

            if payload_file_path.exists():
                content = payload_file_path.read_text()
                if "HAC" in request.name and "ORDER_HAC_FINAL_NEG" not in content:
                    remove(request, "simple hac")
                elif "HAA" in request.name and "<Service>" not in content:
                    remove(request, "empty haa")
                elif "wssparam" in request.name:
                    remove(request, "wssparam")
                else:
                    rm_if_similar(payload_file_path, content)
            elif payload_dir_path.exists():
                for file in payload_dir_path.iterdir():
                    content = file.read_text()
                    rm_if_similar(file, content)
                if not any(payload_dir_path.iterdir()):
                    remove(request, "empty request")
            elif (
                request.name != "fetch"
                and request.name != "submit"
                and not payload_file_path.exists()
                and not payload_dir_path.exists()
            ):
                remove(request, "empty request")

        if not any(date.iterdir()):
            remove(date, "empty dir")



#!/usr/bin/python3
# Update EBICS constants file using latest external code sets files

from io import BytesIO
from zipfile import ZipFile

import polars as pl
import requests


def iso20022codegenExternalCodeSet():
    # Get XLSX zip file from server
    r = requests.get(
        "https://www.iso20022.org/sites/default/files/media/file/ExternalCodeSets_XLSX.zip"
    )
    assert r.status_code == 200

    # Unzip the XLSX file
    zip = ZipFile(BytesIO(r.content))
    files = zip.namelist()
    assert len(files) == 1
    bytes = zip.read(files[0])

    # Parse excel
    df = pl.read_excel(bytes, sheet_name="AllCodeSets")

    def extractCodeSet(setName: str, className: str) -> str:
        out = f"enum class {className}(val isoCode: String, val description: String) {{"

        for row in (
            df.filter(pl.col("Code Set") == setName).sort("Code Value").rows(named=True)
        ):
            (value, isoCode, description) = (
                row["Code Value"],
                row["Code Name"],
                row["Code Definition"].split("\n", 1)[0].strip().replace("_x000D_", ""),
            )
            out += f'\n\t{value}("{isoCode}", "{description}"),'

        out += "\n}"
        return out

    # Write kotlin file
    kt = f"""/*
 * This file is part of LibEuFin.
 * Copyright (C) 2024 Taler Systems S.A.

 * LibEuFin is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation; either version 3, or
 * (at your option) any later version.

 * LibEuFin is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General
 * Public License for more details.

 * You should have received a copy of the GNU Affero General Public
 * License along with LibEuFin; see the file COPYING.  If not, see
 * <http://www.gnu.org/licenses/>
 */

// THIS FILE IS GENERATED, DO NOT EDIT

package tech.libeufin.nexus.iso20022

{extractCodeSet("ExternalStatusReason1Code", "ExternalStatusReasonCode")}

{extractCodeSet("ExternalPaymentGroupStatus1Code", "ExternalPaymentGroupStatusCode")}

{extractCodeSet("ExternalPaymentTransactionStatus1Code", "ExternalPaymentTransactionStatusCode")}

{extractCodeSet("ExternalReturnReason1Code", "ExternalReturnReasonCode")}

"""
    with open(
        "src/main/kotlin/tech/libeufin/nexus/iso20022/ExternalCodeSets.kt", "w"
    ) as file1:
        file1.write(kt)


def iso20022codegenBankTransactionCode():
    # Get XLSX zip file from server
    r = requests.get(
        "https://www.iso20022.org/sites/default/files/media/file/BTC_Codification_21March2024.xlsx"
    )
    assert r.status_code == 200

    # Get the XLSX file
    bytes = r.content

    # Parse excel
    df = pl.read_excel(
        bytes,
        sheet_name="BTC_Codification",
        read_options={"header_row": 2},
    ).rename(lambda name: name.splitlines()[0])

    def extractCodeSet(setName: str, className: str) -> str:
        out = f"enum class {className}(val description: String) {{"
        codeName = f"{setName} Code"
        for row in (
            df.group_by(codeName)
            .agg(pl.col(setName).unique().sort())
            .sort(codeName)
            .rows(named=True)
        ):
            if len(row[setName]) > 1:
                print(row)

            (code, description) = (
                row[codeName].strip().replace("\xa0", ""),
                row[setName][0].strip(),
            )
            out += f'\n\t{code}("{description}"),'

        out += "\n}"
        return out

    # Write kotlin file
    kt = f"""/*
 * This file is part of LibEuFin.
 * Copyright (C) 2024 Taler Systems S.A.

 * LibEuFin is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation; either version 3, or
 * (at your option) any later version.

 * LibEuFin is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General
 * Public License for more details.

 * You should have received a copy of the GNU Affero General Public
 * License along with LibEuFin; see the file COPYING.  If not, see
 * <http://www.gnu.org/licenses/>
 */

// THIS FILE IS GENERATED, DO NOT EDIT

package tech.libeufin.nexus.iso20022

{extractCodeSet("Domain", "ExternalBankTransactionDomainCode")}

{extractCodeSet("Family", "ExternalBankTransactionFamilyCode")}

{extractCodeSet("SubFamily", "ExternalBankTransactionSubFamilyCode")}

"""
    with open(
        "src/main/kotlin/tech/libeufin/nexus/iso20022/BankTransactionCode.kt", "w"
    ) as file1:
        file1.write(kt)


iso20022codegenExternalCodeSet()
iso20022codegenBankTransactionCode()
